﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Emgu.CV;
using Emgu.CV.Structure;
using bbiwarg.Images;
using bbiwarg.Utility;

namespace bbiwarg.Detectors.Fingers
{
    class Finger
    {
        private FingerSliceTrail sliceTrail;
        public Vector2D Tip { get { return sliceTrail.Start.Mid; } }
        public Vector2D Hand { get { return sliceTrail.End.Mid; } }
        public Line2D Line { get { return sliceTrail.Line; } }
        public FingerSliceTrail SliceTrail { get { return sliceTrail; } private set { sliceTrail = value; } }

        public Finger(FingerSliceTrail sliceTrail)
        {
            SliceTrail = sliceTrail;
        }

        public float getSimilarity(Finger compareFinger)
        {
            Line2D compareLine = compareFinger.Line;

            //thresholds
            float thresholdMaxAngle = (float)(30 * Math.PI / 180); // 30°
            float thresholdMaxParallelDistance = 40;
            float thresholdMaxVerticalDistance = 40;

            //check angle
            float angle = Line.getAngleBetween(compareLine);
            float angleSimilarity = Math.Max(1 - angle / thresholdMaxAngle, 0);

            //check parallel distance
            float parallelDistance = Line.getParallelDistanceTo(compareLine);
            float parallelDistanceSimilarity = Math.Max(1 - parallelDistance / thresholdMaxParallelDistance, 0);

            //check vertical distance
            float verticalDistance = Line.getVerticalDistanceTo(compareLine);
            float verticalDistanceSimilarity = Math.Max(1 - verticalDistance / thresholdMaxVerticalDistance, 0);

            return (angleSimilarity + parallelDistanceSimilarity + verticalDistanceSimilarity) / 3;
        }
    }
}
